@@ -26,44 +26,17 @@
 
 from __future__ import annotations
 
-import warnings
 from datetime import datetime, timedelta
 from typing import Optional, List, Union, TYPE_CHECKING, overload
 
 from dateutil.parser import parse
-from pymeos_cffi.functions import pg_timestamp_in, datetime_to_timestamptz, timestampset_end_timestamp, \
-    timestampset_start_timestamp, timestampset_num_timestamps, \
-    timestampset_timestamps, \
-    timestampset_timestamp_n, \
-    timestampset_out, timestamptz_to_datetime, pg_timestamptz_out, timestampset_shift_tscale, timedelta_to_interval, \
-    timestampset_eq, timestampset_ne, timestampset_cmp, timestampset_lt, timestampset_le, timestampset_ge, \
-    timestampset_gt, timestampset_make, timestampset_in, timestampset_hash, timestampset_copy, \
-    timestampset_to_periodset, adjacent_timestampset_period, adjacent_timestampset_periodset, \
-    contained_timestampset_period, contained_timestampset_periodset, contained_timestampset_timestampset, \
-    contains_timestampset_timestamp, contains_timestampset_timestampset, overlaps_timestampset_period, \
-    overlaps_timestampset_periodset, overlaps_timestampset_timestampset, after_timestampset_period, \
-    after_timestampset_periodset, after_timestampset_timestamp, after_timestampset_timestampset, \
-    before_timestampset_period, before_timestampset_periodset, before_timestampset_timestamp, \
-    before_timestampset_timestampset, overbefore_timestampset_timestampset, overbefore_timestampset_timestamp, \
-    overbefore_timestampset_periodset, overbefore_timestampset_period, overafter_timestampset_timestampset, \
-    overafter_timestampset_timestamp, overafter_timestampset_periodset, overafter_timestampset_period, \
-    intersection_timestampset_period, intersection_timestampset_periodset, intersection_timestampset_timestamp, \
-    intersection_timestampset_timestampset, minus_timestampset_period, minus_timestampset_periodset, \
-    minus_timestampset_timestamp, minus_timestampset_timestampset, union_timestampset_period, \
-    union_timestampset_periodset, union_timestampset_timestamp, union_timestampset_timestampset, \
-    distance_timestampset_period, distance_timestampset_periodset, distance_timestampset_timestamp, \
-    distance_timestampset_timestampset, timestampset_timespan, interval_to_timedelta
+from pymeos_cffi import *
 
 if TYPE_CHECKING:
-    # Import here to use in type hints
+    from ..temporal import Temporal
     from .period import Period
     from .periodset import PeriodSet
-
-try:
-    # Do not make psycopg2 a requirement.
-    from psycopg2.extensions import ISQLQuote
-except ImportError:
-    warnings.warn('psycopg2 not installed', ImportWarning)
+    from .time import Time
 
 
 class TimestampSet:
@@ -100,6 +73,14 @@ class TimestampSet:
                      for ts in timestamp_list]
             self._inner = timestampset_make(times, len(times))
 
+    @staticmethod
+    def from_hexwkb(hexwkb: str) -> TimestampSet:
+        result = timestampset_from_hexwkb(hexwkb)
+        return TimestampSet(_inner=result)
+
+    def as_hexwkb(self) -> str:
+        return timestampset_as_hexwkb(self._inner, -1)[0]
+
     @property
     def timespan(self) -> timedelta:
         """
@@ -153,7 +134,8 @@ class TimestampSet:
         tss = timestampset_timestamps(self._inner)
         return [timestamptz_to_datetime(tss[i]) for i in range(self.num_timestamps)]
 
-    def shift_tscale(self, shift_delta: Optional[timedelta] = None, scale_delta: Optional[timedelta] = None) -> TimestampSet:
+    def shift_tscale(self, shift_delta: Optional[timedelta] = None,
+                     scale_delta: Optional[timedelta] = None) -> TimestampSet:
         """
         Shift the timestamp set by a time interval
         """
@@ -169,49 +151,61 @@ class TimestampSet:
         from .periodset import PeriodSet
         return PeriodSet(_inner=timestampset_to_periodset(self._inner))
 
-    def is_adjacent(self, other: Union[Period, PeriodSet]) -> bool:
+    def is_adjacent(self, other: Union[Period, PeriodSet, Temporal]) -> bool:
         from .period import Period
         from .periodset import PeriodSet
+        from ..temporal import Temporal
         if isinstance(other, Period):
             return adjacent_timestampset_period(self._inner, other._inner)
         elif isinstance(other, PeriodSet):
             return adjacent_timestampset_periodset(self._inner, other._inner)
+        elif isinstance(other, Temporal):
+            return adjacent_timestampset_temporal(self._inner, other._inner)
         else:
             raise TypeError(f'Operation not supported with type {other.__class__}')
 
-    def is_contained_in(self, container: Union[Period, PeriodSet, TimestampSet]) -> bool:
+    def is_contained_in(self, container: Union[Period, PeriodSet, TimestampSet, Temporal]) -> bool:
         from .period import Period
         from .periodset import PeriodSet
+        from ..temporal import Temporal
         if isinstance(container, Period):
             return contained_timestampset_period(self._inner, container._inner)
         elif isinstance(container, PeriodSet):
             return contained_timestampset_periodset(self._inner, container._inner)
         elif isinstance(container, TimestampSet):
             return contained_timestampset_timestampset(self._inner, container._inner)
+        elif isinstance(container, Temporal):
+            return contained_timestampset_temporal(self._inner, container._inner)
         else:
             raise TypeError(f'Operation not supported with type {container.__class__}')
 
-    def contains(self, content: Union[datetime, TimestampSet]) -> bool:
+    def contains(self, content: Union[datetime, TimestampSet, Temporal]) -> bool:
+        from ..temporal import Temporal
         if isinstance(content, datetime):
             return contains_timestampset_timestamp(self._inner, datetime_to_timestamptz(content))
         elif isinstance(content, TimestampSet):
             return contains_timestampset_timestampset(self._inner, content._inner)
+        elif isinstance(content, Temporal):
+            return contains_timestampset_temporal(self._inner, content._inner)
         else:
             raise TypeError(f'Operation not supported with type {content.__class__}')
 
-    def overlaps(self, other: Union[Period, PeriodSet, TimestampSet]) -> bool:
+    def overlaps(self, other: Union[Period, PeriodSet, TimestampSet, Temporal]) -> bool:
         from .period import Period
         from .periodset import PeriodSet
+        from ..temporal import Temporal
         if isinstance(other, Period):
             return overlaps_timestampset_period(self._inner, other._inner)
         elif isinstance(other, PeriodSet):
             return overlaps_timestampset_periodset(self._inner, other._inner)
         elif isinstance(other, TimestampSet):
             return overlaps_timestampset_timestampset(self._inner, other._inner)
+        elif isinstance(other, Temporal):
+            return overlaps_timestampset_temporal(self._inner, other._inner)
         else:
             raise TypeError(f'Operation not supported with type {other.__class__}')
 
-    def is_after(self, other: Union[Period, PeriodSet, datetime, TimestampSet]) -> bool:
+    def is_after(self, other: Time) -> bool:
         from .period import Period
         from .periodset import PeriodSet
         if isinstance(other, Period):
@@ -220,12 +214,14 @@ class TimestampSet:
             return after_timestampset_periodset(self._inner, other._inner)
         elif isinstance(other, datetime):
             return after_timestampset_timestamp(self._inner, datetime_to_timestamptz(other))
-        elif isinstance(other, TimestampSet):
+        if isinstance(other, TimestampSet):
             return after_timestampset_timestampset(self._inner, other._inner)
+        elif isinstance(other, Temporal):
+            return after_timestampset_temporal(self._inner, other._inner)
         else:
             raise TypeError(f'Operation not supported with type {other.__class__}')
 
-    def is_before(self, other: Union[Period, PeriodSet, datetime, TimestampSet]) -> bool:
+    def is_before(self, other: Time) -> bool:
         from .period import Period
         from .periodset import PeriodSet
         if isinstance(other, Period):
@@ -234,12 +230,14 @@ class TimestampSet:
             return before_timestampset_periodset(self._inner, other._inner)
         elif isinstance(other, datetime):
             return before_timestampset_timestamp(self._inner, datetime_to_timestamptz(other))
-        elif isinstance(other, TimestampSet):
+        if isinstance(other, TimestampSet):
             return before_timestampset_timestampset(self._inner, other._inner)
+        elif isinstance(other, Temporal):
+            return before_timestampset_temporal(self._inner, other._inner)
         else:
             raise TypeError(f'Operation not supported with type {other.__class__}')
 
-    def is_over_or_after(self, other: Union[Period, PeriodSet, datetime, TimestampSet]) -> bool:
+    def is_over_or_after(self, other: Time) -> bool:
         from .period import Period
         from .periodset import PeriodSet
         if isinstance(other, Period):
@@ -248,12 +246,14 @@ class TimestampSet:
             return overafter_timestampset_periodset(self._inner, other._inner)
         elif isinstance(other, datetime):
             return overafter_timestampset_timestamp(self._inner, datetime_to_timestamptz(other))
-        elif isinstance(other, TimestampSet):
+        if isinstance(other, TimestampSet):
             return overafter_timestampset_timestampset(self._inner, other._inner)
+        elif isinstance(other, Temporal):
+            return overafter_timestampset_temporal(self._inner, other._inner)
         else:
             raise TypeError(f'Operation not supported with type {other.__class__}')
 
-    def is_over_or_before(self, other: Union[Period, PeriodSet, datetime, TimestampSet]) -> bool:
+    def is_over_or_before(self, other: Time) -> bool:
         from .period import Period
         from .periodset import PeriodSet
         if isinstance(other, Period):
@@ -262,12 +262,21 @@ class TimestampSet:
             return overbefore_timestampset_periodset(self._inner, other._inner)
         elif isinstance(other, datetime):
             return overbefore_timestampset_timestamp(self._inner, datetime_to_timestamptz(other))
-        elif isinstance(other, TimestampSet):
+        if isinstance(other, TimestampSet):
             return overbefore_timestampset_timestampset(self._inner, other._inner)
+        elif isinstance(other, Temporal):
+            return overbefore_timestampset_temporal(self._inner, other._inner)
         else:
             raise TypeError(f'Operation not supported with type {other.__class__}')
 
-    def distance(self, other: Union[Period, PeriodSet, datetime, TimestampSet]) -> float:
+    def is_same(self, other: Temporal) -> bool:
+        from ..temporal import Temporal
+        if isinstance(other, Temporal):
+            return same_timestampset_temporal(self._inner, other._inner)
+        else:
+            raise TypeError(f'Operation not supported with type {other.__class__}')
+
+    def distance(self, other: Time) -> float:
         from .period import Period
         from .periodset import PeriodSet
         if isinstance(other, Period):
@@ -289,8 +298,7 @@ class TimestampSet:
     def intersection(self, other: Union[Period, PeriodSet, TimestampSet]) -> TimestampSet:
         ...
 
-    def intersection(self, other: Union[Period, PeriodSet, datetime, TimestampSet]) -> \
-            Union[datetime, TimestampSet]:
+    def intersection(self, other: Time) -> Union[datetime, TimestampSet]:
         from .period import Period
         from .periodset import PeriodSet
         if isinstance(other, Period):
@@ -305,7 +313,7 @@ class TimestampSet:
         else:
             raise TypeError(f'Operation not supported with type {other.__class__}')
 
-    def minus(self, other: Union[Period, PeriodSet, datetime, TimestampSet]) -> TimestampSet:
+    def minus(self, other: Time) -> TimestampSet:
         from .period import Period
         from .periodset import PeriodSet
         if isinstance(other, Period):
@@ -327,7 +335,7 @@ class TimestampSet:
     def union(self, other: Union[datetime, TimestampSet]) -> TimestampSet:
         ...
 
-    def union(self, other: Union[Period, PeriodSet, datetime, TimestampSet]) -> Union[PeriodSet, TimestampSet]:
+    def union(self, other: Time) -> Union[PeriodSet, TimestampSet]:
         from .period import Period
         from .periodset import PeriodSet
         if isinstance(other, Period):
@@ -388,18 +396,8 @@ class TimestampSet:
             return timestampset_gt(self._inner, other._inner)
         raise TypeError(f'Operation not supported with type {other.__class__}')
 
-    # Psycopg2 interface.
-    def __conform__(self, protocol):
-        if protocol is ISQLQuote:
-            return self
-
-    def getquoted(self):
-        return "{}".format(self.__str__())
-
-    # End Psycopg2 interface.
-
     @staticmethod
-    def read_from_cursor(value, cursor=None):
+    def read_from_cursor(value, _=None):
         if not value:
             return None
         return TimestampSet(string=value)
@@ -417,3 +415,7 @@ class TimestampSet:
     def __repr__(self):
         return (f'{self.__class__.__name__}'
                 f'({self})')
+
+    def plot(self, *args, **kwargs):
+        from ..plotters import TimePlotter
+        return TimePlotter.plot_timestampset(self, *args, **kwargs)
